==> pulseaudio_0.9.10.orig.tar.gz: pulseaudio-0.9.10/src/pulsecore/hook-list.c <==
/* $Id: hook-list.c 1971 2007-10-28 19:13:50Z lennart $ */

/***
  This file is part of PulseAudio.

  Copyright 2006 Lennart Poettering

  PulseAudio is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  PulseAudio is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with PulseAudio; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA.
***/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <pulsecore/macro.h>

#include "hook-list.h"

void pa_hook_init(pa_hook *hook, void *data) {
    pa_assert(hook);

    PA_LLIST_HEAD_INIT(pa_hook_slot, hook->slots);
    hook->last = NULL;
    hook->n_dead = hook->firing = 0;
    hook->data = data;
}

static void slot_free(pa_hook *hook, pa_hook_slot *slot) {
    pa_assert(hook);
    pa_assert(slot);

    if (hook->last == slot)
        hook->last = slot->prev;

    PA_LLIST_REMOVE(pa_hook_slot, hook->slots, slot);

    pa_xfree(slot);
}

void pa_hook_free(pa_hook *hook) {
    pa_assert(hook);
    pa_assert(!hook->firing);

    while (hook->slots)
        slot_free(hook, hook->slots);

    pa_hook_init(hook, NULL);
}

pa_hook_slot* pa_hook_connect(pa_hook *hook, pa_hook_cb_t cb, void *data) {
    pa_hook_slot *slot;

    pa_assert(cb);

    slot = pa_xnew(pa_hook_slot, 1);
    slot->hook = hook;
    slot->dead = 0;
    slot->callback = cb;
    slot->data = data;

    PA_LLIST_INSERT_AFTER(pa_hook_slot, hook->slots, hook->last, slot);
    hook->last = slot;

    return slot;
}

void pa_hook_slot_free(pa_hook_slot *slot) {
    pa_assert(slot);
    pa_assert(!slot->dead);

    if (slot->hook->firing > 0) {
        slot->dead = 1;
        slot->hook->n_dead++;
    } else
        slot_free(slot->hook, slot);
}

pa_hook_result_t pa_hook_fire(pa_hook *hook, void *data) {
    pa_hook_slot *slot, *next;
    pa_hook_result_t result = PA_HOOK_OK;

    pa_assert(hook);

    hook->firing ++;

    for (slot = hook->slots; slot; slot = slot->next) {
        if (slot->dead)
            continue;

        if ((result = slot->callback(hook->data, data, slot->data)) != PA_HOOK_OK)
            break;
    }

    hook->firing --;

    for (slot = hook->slots; hook->n_dead > 0 && slot; slot = next) {
        next = slot->next;

        if (slot->dead) {
            slot_free(hook, slot);
            hook->n_dead--;
        }
    }

    return result;
}

==> pulseaudio_0.9.14.orig.tar.gz: pulseaudio-0.9.14/src/pulsecore/hook-list.c <==
/***
  This file is part of PulseAudio.

  Copyright 2006 Lennart Poettering

  PulseAudio is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.

  PulseAudio is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with PulseAudio; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA.
***/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <pulsecore/macro.h>

#include "hook-list.h"

void pa_hook_init(pa_hook *hook, void *data) {
    pa_assert(hook);

    PA_LLIST_HEAD_INIT(pa_hook_slot, hook->slots);
    hook->n_dead = hook->n_firing = 0;
    hook->data = data;
}

static void slot_free(pa_hook *hook, pa_hook_slot *slot) {
    pa_assert(hook);
    pa_assert(slot);

    PA_LLIST_REMOVE(pa_hook_slot, hook->slots, slot);

    pa_xfree(slot);
}

void pa_hook_done(pa_hook *hook) {
    pa_assert(hook);
    pa_assert(hook->n_firing == 0);

    while (hook->slots)
        slot_free(hook, hook->slots);

    pa_hook_init(hook, NULL);
}

pa_hook_slot* pa_hook_connect(pa_hook *hook, pa_hook_priority_t prio, pa_hook_cb_t cb, void *data) {
    pa_hook_slot *slot, *where, *prev;

    pa_assert(cb);

    slot = pa_xnew(pa_hook_slot, 1);
    slot->hook = hook;
    slot->dead = FALSE;
    slot->callback = cb;
    slot->data = data;
    slot->priority = prio;

    prev = NULL;
    for (where = hook->slots; where; where = where->next) {
        if (prio < where->priority)
            break;
        prev = where;
    }

    PA_LLIST_INSERT_AFTER(pa_hook_slot, hook->slots, prev, slot);

    return slot;
}

void pa_hook_slot_free(pa_hook_slot *slot) {
    pa_assert(slot);
    pa_assert(!slot->dead);

    if (slot->hook->n_firing > 0) {
        slot->dead = TRUE;
        slot->hook->n_dead++;
    } else
        slot_free(slot->hook, slot);
}

pa_hook_result_t pa_hook_fire(pa_hook *hook, void *data) {
    pa_hook_slot *slot, *next;
    pa_hook_result_t result = PA_HOOK_OK;

    pa_assert(hook);

    hook->n_firing ++;

    for (slot = hook->slots; slot; slot = slot->next) {
        if (slot->dead)
            continue;

        if ((result = slot->callback(hook->data, data, slot->data)) != PA_HOOK_OK)
            break;
    }

    hook->n_firing --;
    pa_assert(hook->n_firing >= 0);

    for (slot = hook->slots; hook->n_dead > 0 && slot; slot = next) {
        next = slot->next;

        if (slot->dead) {
            slot_free(hook, slot);
            hook->n_dead--;
        }
    }

    pa_assert(hook->n_dead == 0);

    return result;
}
